@@ -19,7 +19,6 @@ This module defines all possible packages of the Scalyr Agent and how they can b
 
 import json
 import pathlib as pl
-import shlex
 import tarfile
 import abc
 import shutil
@@ -37,7 +36,7 @@ from agent_build.tools import constants
 from agent_build.tools.environment_deployments import deployments
 from agent_build.tools import build_in_docker
 from agent_build.tools import common
-
+from agent_build.tools.common import shlex_join
 
 __PARENT_DIR__ = pl.Path(__file__).absolute().parent
 __SOURCE_ROOT__ = __PARENT_DIR__.parent
@@ -263,7 +262,7 @@ class PackageBuilder(abc.ABC):
             "--locally",
         ]
 
-        command = shlex.join(command_args)  # pylint: disable=no-member
+        command = shlex_join(command_args)  # pylint: disable=no-member
 
         # Run the docker build inside the result image of the deployment.
         base_image_name = self.deployment.result_image_name.lower()
@@ -344,15 +343,30 @@ class PackageBuilder(abc.ABC):
 
     @staticmethod
     def _add_config(
-        config_source_path: Union[str, pl.Path], output_path: Union[str, pl.Path]
+        base_config_source_path: Union[str, pl.Path],
+        output_path: Union[str, pl.Path],
+        additional_config_paths: List[pl.Path] = None,
     ):
         """
         Copy config folder from the specified path to the target path.
+        :param base_config_source_path: Path to the base config directory to copy.
+        :param additional_config_paths: List of paths to config directories to copy to the result config directory.
+            New files are places along with files from the 'base_config_source_path' or overwrite them.
         """
-        config_source_path = pl.Path(config_source_path)
+        base_config_source_path = pl.Path(base_config_source_path)
         output_path = pl.Path(output_path)
         # Copy config
-        shutil.copytree(config_source_path, output_path)
+        shutil.copytree(base_config_source_path, output_path)
+
+        # Copy additional config files.
+        for a_config_path in additional_config_paths or []:
+            # pylint: disable=unexpected-keyword-arg
+            shutil.copytree(
+                a_config_path,
+                output_path,
+                dirs_exist_ok=True,
+            )
+            # pylint: enable=unexpected-keyword-arg
 
         # Make sure config file has 640 permissions
         config_file_path = output_path / "agent.json"
@@ -722,6 +736,7 @@ class ContainerPackageBuilder(LinuxFhsBasedPackageBuilder):
         base_image_deployment_step_cls: Type[deployments.BuildDockerBaseImageStep],
         variant: str = None,
         no_versioned_file_name: bool = False,
+        additional_config_paths: List[pl.Path] = None,
     ):
         """
         :param config_path: Path to the configuration directory which will be copied to the image.
@@ -729,6 +744,8 @@ class ContainerPackageBuilder(LinuxFhsBasedPackageBuilder):
         tweak to the name is required. This is used to produce different packages even for the same package type (such
         as 'rpm').
         :param no_versioned_file_name:  True if the version number should not be embedded in the artifact's file name.
+        :param additional_config_paths: List of paths to config directories to copy to the result config directory.
+            New files are placed along with files from the 'config_path' or overwrite them.
         """
         self._name = name
         self.dockerfile_path = __SOURCE_ROOT__ / "agent_build/docker/Dockerfile"
@@ -743,6 +760,7 @@ class ContainerPackageBuilder(LinuxFhsBasedPackageBuilder):
         self.base_image_deployment_step_cls = base_image_deployment_step_cls
 
         self.config_path = config_path
+        self.additional_config_paths = additional_config_paths or []
 
     @property
     def name(self) -> str:
@@ -758,8 +776,9 @@ class ContainerPackageBuilder(LinuxFhsBasedPackageBuilder):
 
         # Copy config
         self._add_config(
-            config_source_path=self.config_path,
+            base_config_source_path=self.config_path,
             output_path=self._package_root_path / "etc/scalyr-agent-2",
+            additional_config_paths=self.additional_config_paths,
         )
 
     def _build(self):
@@ -1058,6 +1077,15 @@ class K8sWithOpenMetricsMonitorPackageBuilder(K8sPackageBuilder):
     RESULT_IMAGE_NAMES = ["scalyr-k8s-agent-with-openmetrics-monitor"]
 
 
+class K8sRestartAgentOnMonitorsDeath(K8sPackageBuilder):
+    """
+    An image with enabled feature to shut down the agent if kubernetes monitor is also down.
+    """
+
+    PACKAGE_TYPE = constants.PackageType.K8S
+    RESULT_IMAGE_NAMES = ["scalyr-k8s-restart-agent-on-monitor-death"]
+
+
 class DockerJsonPackageBuilder(ContainerPackageBuilder):
     """
     An image for running on Docker configured to fetch logs via the file system (the container log
@@ -1157,3 +1185,21 @@ K8S_CONTAINER_WITH_OPENMETRICS_MONITOR_BUILDER_ALPINE = (
         base_image_deployment_step_cls=deployments.BuildAlpineDockerBaseImageStep,
     )
 )
+
+# Those builds for a temporary testing of the "stop_agent_on_failure" option for the k8s image build.
+K8S_CONTAINER_RESTART_AGENT_ON_MONITOR_DEATH_DEBIAN = K8sRestartAgentOnMonitorsDeath(
+    name="k8s-restart-agent-on-monitor-death-debian",
+    config_path=_CONFIGS_PATH / "k8s-config",
+    base_image_deployment_step_cls=deployments.BuildDebianDockerBaseImageStep,
+    additional_config_paths=[
+        _CONFIGS_PATH / "k8s-config-restart-agent-on-monitor-death"
+    ],
+)
+K8S_CONTAINER_RESTART_AGENT_ON_MONITOR_DEATH_ALPINE = K8sRestartAgentOnMonitorsDeath(
+    name="k8s-restart-agent-on-monitor-death-alpine",
+    config_path=_CONFIGS_PATH / "k8s-config",
+    base_image_deployment_step_cls=deployments.BuildAlpineDockerBaseImageStep,
+    additional_config_paths=[
+        _CONFIGS_PATH / "k8s-config-restart-agent-on-monitor-death"
+    ],
+)
